<?php

include_once(dirname(__DIR__).'/vendor/autoload.php');
include_once($_ENV["EDICONNECT_EBP"]."Functions/Logs.php");

use Automattic\WooCommerce\Client;

$dotenv = Dotenv\Dotenv::createImmutable(dirname(__DIR__));
$dotenv->load();

class APIAuth
{
    public function __construct(){

        $this->folderId = $_ENV['FOLDER_ID'];
        $this->ebp_subscription_key = $_ENV['EBP_SUBSCRIPTION_KEY'];
        $this->authorize_url = $_ENV['AUTHORIZE_URL'];
        $this->token_url = $_ENV['TOKEN_URL'];
        $this->callback_uri = $_ENV['CALLBACK_URI'];
        $this->client_id = $_ENV['CLIENT_ID'];
        $this->client_secret = $_ENV['CLIENT_SECRET'];      

        $this->root_dir = $_ENV["EDICONNECT_EBP"];

        $this->exportField = $_ENV['EXPORT_FIELD'];
        $this->itemFamilyField = $_ENV['ITEM_FAMILY_FIELD'];
        $access_file = json_decode(file_get_contents($this->root_dir."Configurations/access.json"), true);
        
        $this->access_token = $access_file["access_token"];
        $this->refresh_token = $access_file["refresh_token"];


        if (isset($this->refresh_token) && $this->refresh_token !== "") {
            $tokens = $this->getAccessTokenFromRefreshToken($this->refresh_token);
            $this->access_token = $tokens["access_token"];
            $this->refresh_token = $tokens["refresh_token"];
        }else{
            if (isset($_POST["authorization_code"])) {
                $tokens = $this->getAccessToken($_POST["authorization_code"]);
                $this->access_token = $tokens["access_token"];
                $this->refresh_token = $tokens["refresh_token"];
            } elseif (isset($_GET["code"])) {
                $tokens = $this->getAccessToken($_GET["code"]);
                $this->access_token = $tokens["access_token"];
                $this->refresh_token = $tokens["refresh_token"];
            } else {
                $this->getAuthorizationCode();
            }
        }

        $this->headers = array(
            "Cache-Control: no-cache",
            "ebp-subscription-key: $this->ebp_subscription_key",
            "Authorization: Bearer $this->access_token"
        );


        //Export
        $this->url_api_invoices_export = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=10000&DocumentType=Facture";
        $this->url_api_orders_export = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=10000&DocumentType=Commande";
        $this->url_api_desadv_export = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=10000&DocumentType=BonLivraison";
        $this->url_api_avoir_export = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=10000&DocumentType=Avoir";
        
        //Import
        $this->url_api_orders_import = "https://api-developpeurs.ebp.com/import/api/v1/createImport?folderId=$this->folderId&ImportCategory=SaleOrders&ImportSettings=".$_ENV['ORDERS_IMPORT_SETTINGS'];
        $this->url_api_invoices_import = "https://api-developpeurs.ebp.com/import/api/v1/createImport?folderId=$this->folderId&ImportCategory=SaleInvoices&ImportSettings=".$_ENV['INVOICES_IMPORT_SETTINGS'];
        $this->url_api_desadv_import = "https://api-developpeurs.ebp.com/import/api/v1/createImport?folderId=$this->folderId&ImportCategory=SaleDelivery&ImportSettings=".$_ENV['DESADV_IMPORT_SETTINGS'];
        $this->url_api_customer_import = "https://api-developpeurs.ebp.com/import/api/v1/createImport?folderId=$this->folderId&ImportCategory=Customers&ImportSettings=".$_ENV['CLIENTS_IMPORT_SETTINGS'];
        
    }
    //Export url functions
    public function GetAuthHeader(){ return $this->headers;} 
    public function GetApiPathInvoicesExport(){ return $this->url_api_invoices_export;} 
    public function GetApiPathOrdersExport(){ return $this->url_api_orders_export;} 
    public function GetApiPathDesadvExport(){ return $this->url_api_desadv_export;} 
    public function GetApiPathCustomerImport(){ return $this->url_api_customer_import;}
    //Import url functions
    public function GetApiPathInvoiceImport(){ return $this->url_api_invoices_import;} 
    public function GetApiPathDesadvImport(){ return $this->url_api_desadv_import;} 
    public function GetApiPathOrderImport(){ return $this->url_api_orders_import;} 

    public function getApiPathAllInvoicesExport($offset=0){ 
        // Last one $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=45000&DocumentType=Facture&limit=500&offset=$offset";
        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=45000&DocumentType=Facture&limit=500&WhereCondition=%20%20type%3A%20CustomFilter%0A%20%20column%3A%20DocumentDate%0A%20%20operator%3A%20Between%0A%20%20valueType%20%3A%20String%0A%20%20value%3A%0A%20%20-%20%272023-05-30T00%3A00%3A00%27%0A%20%20-%20%272023-06-10T00%3A00%3A00%27";
        
        return $url;
    }
    public function getApiPathAllCreditNotesExport($offset=0){ 
        //$url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=10000&limit=500&offset=$offset&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20NumberPrefix%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27AV%27";
        // Last one $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=45000&DocumentType=Avoir&limit=5&offset=$offset";

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=45000&DocumentType=Avoir&limit=500&WhereCondition=%20%20type%3A%20CustomFilter%0A%20%20column%3A%20DocumentDate%0A%20%20operator%3A%20Between%0A%20%20valueType%20%3A%20String%0A%20%20value%3A%0A%20%20-%20%272023-05-31T00%3A00%3A00%27%0A%20%20-%20%272023-06-10T00%3A00%3A00%27";

        return $url;
    }

    public function getApiPathAllDevisExport($offset=0){ 
        // $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=10000&limit=500&offset=$offset&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20NumberPrefix%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27DV%27";

        // last one $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=45000&DocumentType=Devis&limit=100&offset=$offset";
        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument/WithLines?Duration=45000&DocumentType=Devis&limit=500&WhereCondition=%20%20type%3A%20CustomFilter%0A%20%20column%3A%20DocumentDate%0A%20%20operator%3A%20Between%0A%20%20valueType%20%3A%20String%0A%20%20value%3A%0A%20%20-%20%272023-05-31T00%3A00%3A00%27%0A%20%20-%20%272023-06-10T00%3A00%3A00%27";
        
        return $url;
    }

    public function getApiPathAllInvoicesStatExport($offset=0){ 
        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument?Duration=45000&DocumentType=Facture&Columns=Id,DocumentNumber,ValidationState,DocumentState,DocumentDate,SettlementModeId&Limit=500&offset=$offset";
        return $url;
    }
    
    public function GetApiPathToCheckIfOrderExist($reference){ 
        $fields = [
            "Id",
            "DocumentNumber"
        ];

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Documents/SaleDocument?Duration=1000&DocumentType=Commande&Columns=". implode($fields, ",") ."&Limit=1&WhereCondition=%20%20type%3A%20CustomFilter%0A%20%20column%3A%20Reference%0A%20%20operator%3A%20Like%0A%20%20valueType%20%3A%20String%0A%20%20value%3A%0A%20%20-%20%27%25%20$reference%27";

        return $url;
    }

    public function GetApiPathCustomersListToExport($offset=0){ 
        $fields = [
            "Id",
            "MainInvoicingContact_Email",
            "sysModifiedDate"
        ];

        $exportFieldValue = 1;

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Clients/ExtendedQuery?Columns=". implode($fields, ",") ."&limit=500&offset=$offset&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20$this->exportField%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$exportFieldValue%27";

        return $url;
    }

    public function GetApiPathCustomersExport($customerId){ 
        $fields = [
            "Id",
            "name",
            "sysCreatedDate",
            "MainDeliveryAddress_Address1",
            "MainDeliveryAddress_City",
            "MainDeliveryAddress_ZipCode",
            "MainDeliveryAddress_CountryIsoCode",
            "MainDeliveryContact_FirstName",
            "MainDeliveryContact_Name",
            "MainDeliveryContact_Email",
            "MainDeliveryContact_Phone",
            "MainInvoicingAddress_Address1",
            "MainInvoicingAddress_City",
            "MainInvoicingAddress_ZipCode",
            "MainInvoicingAddress_CountryIsoCode",
            "MainInvoicingContact_FirstName",
            "MainInvoicingContact_Name",
            "MainInvoicingContact_Email",
            "MainInvoicingContact_Phone",
            "sysModifiedDate"
        ];
        $exportFieldValue = 1;

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Clients/ExtendedQuery?Columns=". implode($fields, ",") ."&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20Id%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$customerId%27";
        return $url;
    }
    
    public function getApiPathAllCustomersToExport($offset=0){ 


        // $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Clients/ExtendedQuery?limit=5";        
        
        // last one $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Clients/ExtendedQuery?Duration=45000&limit=100&offset=$offset";  
        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Clients/ExtendedQuery?Duration=50000&limit=500&WhereCondition=%20%20type%3A%20CustomFilter%0A%20%20column%3A%20sysCreatedDate%0A%20%20operator%3A%20Between%0A%20%20valueType%20%3A%20String%0A%20%20value%3A%0A%20%20-%20%272023-05-25T00%3A00%3A00%27%0A%20%20-%20%272023-06-10T00%3A00%3A00%27";  

       
        // $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Clients?Duration=45000&Limit=100&offset=$offset";
        return $url;
    }

    public function GetApiPathProductExport($id){ 
        $fields = [
            "Item.Id",
            "Item.BarCode",
            "Item.Caption",
            "Item.DesComClear",
            "Item.sysCreatedDate",
            "Item.SalePriceVatIncluded",
            "Item.ItemImage",
            // "ItemFamily.xx_Categorie_sur_wordpress",
            "Item.VirtualStock",
            "Item.UnitId",
            "Item.sysModifiedDate"
        ];

        $exportFieldValue = 1;

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Items/ItemsExtentedQuery?Columns=". implode($fields, ",") ."&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20Item.Id%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$id%27";

        return $url;
    }

    public function GetApiPathProductsListToExport($offset=0){ 
        $fields = [
            "Id",
            "sysModifiedDate"
        ];

        $exportFieldValue = 1;

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Items/ItemsExtentedQuery?Columns=". implode($fields, ",") ."&limit=500&offset=$offset&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20$this->exportField%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$exportFieldValue%27";
        echo $url.'<br>';
        return $url;
    }

    public function getApiPathAllProductsToExport($offset=0){ 
        $exportFieldValue = 1;

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Items/ItemsExtentedQuery?Duration=45000&limit=100&offset=$offset";
        // $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Items/ItemsExtentedQuery?limit=5";

        return $url;
    }


    public function GetApiPathProductsListToDelete($offset=0){ 
        $fields = [
            "Id",
            "sysModifiedDate"
        ];

        $exportFieldValue = 0;

        $url = "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/$this->folderId/Items/ItemsExtentedQuery?Columns=". implode($fields, ",") ."&limit=500&offset=$offset&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20$this->exportField%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$exportFieldValue%27";
        
        return $url;
    }

    public function getAuthorizationCode() {
        $authorization_redirect_url = $this->authorize_url . "?response_type=code&client_id=" . $this->client_id . "&redirect_uri=" . $this->callback_uri . "&scope=openid%20profile%20offline_access";


         header("Location: " . $authorization_redirect_url);
    }

    public function getAccessToken($authorization_code) {
        $authorization = base64_encode("$this->client_id:$this->client_secret");
        $header = array("Authorization: Basic $authorization","Content-Type: application/x-www-form-urlencoded");
        $content = "grant_type=authorization_code&code=$authorization_code&redirect_uri=$this->callback_uri";

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $this->token_url,
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $content
        ));
        
        $response = curl_exec($curl);
        curl_close($curl);
        
        $logs = new Logs();
        $date = date("Y-m-d");

        $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Connexion/Errors/ERROR_CONNEXION_$date.txt";
        if ($response === false ) {
            echo "Failed";
            $file = fopen($error_path, "a");
            $logs->addLog($file, "CONNEXION API EBP ". __LINE__, curl_error($curl), "ERROR");
            die();
        } elseif (!isset(json_decode($response)->access_token)) {
            echo "Error:<br />";
            $file = fopen($error_path, "a");
            $logs->addLog($file, "CONNEXION API EBP ". __LINE__, json_encode(json_decode($response)), "ERROR");
            die();
        }elseif (json_decode($response)->access_token == null) {
            echo "Error:<br />";
            $file = fopen($error_path, "a");
            $logs->addLog($file, "CONNEXION API EBP ". __LINE__, json_encode(json_decode($response)), "ERROR");
            die();
        }
        $response = json_decode($response);

        $data = [
            'access_token' => $response->access_token,
            'refresh_token' => $response->refresh_token,
        ];

        file_put_contents($this->root_dir."Configurations/access.json", json_encode($data, JSON_PRETTY_PRINT));
        return $data;
    }

    public function getAccessTokenFromRefreshToken($refresh_token) {
        $content = "grant_type=refresh_token&refresh_token=$refresh_token&client_id=$this->client_id&client_secret=$this->client_secret";
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $this->token_url,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $content
        ));
        $response = curl_exec($curl);
        curl_close($curl);

        $logs = new Logs();
        $date = date("Y-m-d");

        
        $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Connexion/Errors/ERROR_CONNEXION_$date.txt";
        if ($response === false ) {
            echo "Failed : " . "CONNEXION API EBP ". __LINE__, curl_error($curl);
            $file = fopen($error_path, "a");
            $logs->addLog($file, "CONNEXION API EBP ". __LINE__, curl_error($curl), "ERROR");
            die();
        } elseif (!isset(json_decode($response)->access_token)) {
            echo "Error:<br />". "CONNEXION API EBP ". __LINE__, json_encode(json_decode($response));
            $file = fopen($error_path, "a");
            $logs->addLog($file, "CONNEXION API EBP ". __LINE__, json_encode(json_decode($response)), "ERROR");
            die();
        }elseif (json_decode($response)->access_token == null) {
            echo "Error:<br />". "CONNEXION API EBP ". __LINE__, json_encode(json_decode($response));
            $file = fopen($error_path, "a");
            $logs->addLog($file, "CONNEXION API EBP ". __LINE__, json_encode(json_decode($response)), "ERROR");
            die();
        }

        $response = json_decode($response);
        $data = [
            'access_token' => $response->access_token,
            'refresh_token' => $response->refresh_token,
        ];
        file_put_contents($this->root_dir."Configurations/access.json", json_encode($data, JSON_PRETTY_PRINT));

        return $data;
    }
}
?>