<?php

include_once($_ENV["EDICONNECT_EBP"]."Functions/Logs.php");
include_once($_ENV["EDICONNECT_EBP"]."Functions/Functions.php");
include_once($_ENV["EDICONNECT_EBP"]."Configurations/api.php");
include_once($_ENV["EDICONNECT_EBP"]."Class/queries.class.php");

class ProductsController
{
    
    public function __construct(){
        $this->APIAuth = new APIAuth();

        $this->access_token = $this->APIAuth->access_token;
        $this->ebp_subscription_key = $this->APIAuth->ebp_subscription_key;
    }

    public function productsListToDelete(){
        $logs = new Logs();

        try {
            $date = date("YmdHis");
            $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Deletes/Errors/ERROR_PRODUCT_$date.txt";

            $offset = 0;
            $products = [];

    
            do {
                $url = $this->APIAuth->GetApiPathProductsListToDelete($offset);
                $curl = curl_init();
        
                curl_setopt_array($curl, array(
                    CURLOPT_URL => $url,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => "",
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => "GET",
                    CURLOPT_HTTPHEADER => array(
                        "ebp-subscription-key: $this->ebp_subscription_key",
                        "Authorization: Bearer $this->access_token"
                    ),
                ));
        
                $resp = json_decode(curl_exec($curl),true);
                $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                curl_close($curl);

                if ($http_code == "200") {
                    $products = array_merge($products, $resp["results"]);
                }else{ 
                    echo "Error : ".$http_code." <br> Please check your access TOKEN KEY's to the API.";
        
                    $target_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Deletes/Errors/ERROR_PRODUCT_$date.txt";
                    $file = fopen($error_path, "a");
                    $logs->addLog($file, "SUPPRESSION PRODUIT", "Erreur avec la requete cURL lors de l'export de produits (Erreur $http_code)", "ERROR");
                }

                $offset += 500;
            } while ($resp["paging"]["total"] > 0);

            if (count($products) > 0) {
                $queries = new Queries();
    

                foreach ($products as $product) {
                    $result = $queries->checkIsItemActive("products_syncro", "product_id", $product["Id"]);

                    if ($result) {
                        $isFileGenerated = $this->generateDeleteFile($product);
                        if ($isFileGenerated) {
                            $message = "Fichier de suppression généré pour le produit ".$product["Id"];

                            $success_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Deletes/SUCCESS_PRODUCT_".$product["Id"]."_$date.txt";
                            $file = fopen($success_path, "a");
                            $logs->addLog($file, "SUPPRESSION PRODUIT", $message, "SUCCESS");
                        }else {
                            $message = "Fichier de suppression non généré pour le produit ".$product["Id"];
                            $file = fopen($error_path, "a");
                            $logs->addLog($file, "SUPPRESSION PRODUIT", $message, "ERROR");
                        }
                    }
                }
    
                require_once($_ENV["WORDPRESS_API"]."products/delete.php");
            }   

        } catch (\Throwable $th) {
            //throw $th;
            $date = date("YmdHis");
    
            $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
            $file = fopen($error_path, "a");
            $logs->addLog($file, "SUPPRESSION PRODUIT", $th->getMessage(), "ERROR");

        }
    } 

    public function generateDeleteFile($product){

        try {
            include_once($_ENV["EDICONNECT_EBP"]."Models/ProductsModel.php");
            $ProductModel = new ProductModel();
            
            return $ProductModel->Delete($product); 
        } catch (\Throwable $th) {
            $date = date("YmdHis");
    
            $target_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
            $file = fopen($target_path, "a");
            $logs->addLog($file, "SUPPRESSION PRODUIT", $th->getMessage(), "ERROR");
            return false;
        }

    } 

    public function productsListToExport(){
        $logs = new Logs();

        try {
            $date = date("YmdHis");
            $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";

            $offset = 0;
            $products = [];

    
            do {
                $url = $this->APIAuth->GetApiPathProductsListToExport($offset);
                $curl = curl_init();
        
                curl_setopt_array($curl, array(
                    CURLOPT_URL => $url,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => "",
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => "GET",
                    CURLOPT_HTTPHEADER => array(
                        "ebp-subscription-key: $this->ebp_subscription_key",
                        "Authorization: Bearer $this->access_token"
                    ),
                ));
        
                $resp = json_decode(curl_exec($curl),true);
                $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                curl_close($curl);

                if ($http_code == "200") {
                    $products = array_merge($products, $resp["results"]);
                }else{ 
                    echo "Error : ".$http_code." <br> Please check your access TOKEN KEY's to the API.";
        
                    $target_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
                    $file = fopen($error_path, "a");
                    $logs->addLog($file, "EXPORT PRODUIT", "Erreur avec la requete cURL lors de l'export de produits (Erreur $http_code)", "ERROR");
                }
                $offset += 500;
            } while ($resp["paging"]["total"] > 0);

            if (count($products) > 0) {
                $queries = new Queries();

                foreach ($products as $product) {
                    $result = $queries->checkSysModifiedDateItem("products_syncro", "product_id", $product["Id"]);

                    if (!$result) {
                        $isFileGenerated = $this->Export($product["Id"]);

                        if ($isFileGenerated) {
                            $message = "Fichier d'export généré pour le produit ".$product["Id"];

                            $success_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/SUCCESS_PRODUCT_".$product["Id"]."_$date.txt";
                            $file = fopen($success_path, "a");
                            $logs->addLog($file, "EXPORT PRODUIT", $message, "SUCCESS");
                        }else {
                            $message = "Fichier d'export non généré pour le produit ".$product["Id"];

                            $file = fopen($error_path, "a");
                            $logs->addLog($file, "EXPORT PRODUIT", $message, "ERROR");

                        }
                    }

                    if ($result && $product["sysModifiedDate"] !== $result) {

                        $isFileGenerated = $this->Export($product["Id"]);
                        if ($isFileGenerated) {
                            $message = "Fichier d'export généré pour le produit ".$product["Id"];
                            $success_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/SUCCESS_PRODUCT_".$product["Id"]."_$date.txt";
                            $file = fopen($success_path, "a");
                            $logs->addLog($file, "EXPORT PRODUIT", $message, "SUCCESS");
                        }else {
                            $message = "Fichier d'export non généré pour le produit ".$product["Id"];

                            $file = fopen($error_path, "a");
                            $logs->addLog($file, "EXPORT PRODUIT", $message, "ERROR");

                        }
                    }

                    $queries->setActiveItemField("products_syncro", "product_id", $product["Id"], 1);
                }

                require_once($_ENV["WORDPRESS_API"]."products/export.php");
            }   

        } catch (\Throwable $th) {
            //throw $th;
            $date = date("YmdHis");
    
            $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
            $file = fopen($error_path, "a");
            $logs->addLog($file, "EXPORT PRODUIT", $th->getMessage(), "ERROR");

        }
    } 


    public function exportAllProducts(){
        $logs = new Logs();

        try {
            $date = date("YmdHis");
            $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";

            $offset = 0;
            $products = [];

            $url = $this->APIAuth->getApiPathAllProductsToExport($offset);
            $curl = curl_init();
    
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => "GET",
                CURLOPT_HTTPHEADER => array(
                    "ebp-subscription-key: $this->ebp_subscription_key",
                    "Authorization: Bearer $this->access_token"
                ),
            ));
    
            $resp = json_decode(curl_exec($curl),true);
            $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            if ($http_code == "200") {
                $products = array_merge($products, $resp["results"]);
            }else{ 
                echo "Error : ".$http_code." <br> Please check your access TOKEN KEY's to the API.";
    
                $target_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
                $file = fopen($error_path, "a");
                $logs->addLog($file, "EXPORT PRODUIT", "Erreur avec la requete cURL lors de l'export de produits (Erreur $http_code)", "ERROR");
            }    
            // do {
            //     $url = $this->APIAuth->GetApiPathProductsListToExport($offset);
            //     $curl = curl_init();
        
            //     curl_setopt_array($curl, array(
            //         CURLOPT_URL => $url,
            //         CURLOPT_RETURNTRANSFER => true,
            //         CURLOPT_ENCODING => "",
            //         CURLOPT_MAXREDIRS => 10,
            //         CURLOPT_TIMEOUT => 0,
            //         CURLOPT_FOLLOWLOCATION => true,
            //         CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            //         CURLOPT_CUSTOMREQUEST => "GET",
            //         CURLOPT_HTTPHEADER => array(
            //             "ebp-subscription-key: $this->ebp_subscription_key",
            //             "Authorization: Bearer $this->access_token"
            //         ),
            //     ));
        
            //     $resp = json_decode(curl_exec($curl),true);
            //     $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            //     curl_close($curl);

            //     if ($http_code == "200") {
            //         $products = array_merge($products, $resp["results"]);
            //     }else{ 
            //         echo "Error : ".$http_code." <br> Please check your access TOKEN KEY's to the API.";
        
            //         $target_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
            //         $file = fopen($error_path, "a");
            //         $logs->addLog($file, "EXPORT PRODUIT", "Erreur avec la requete cURL lors de l'export de produits (Erreur $http_code)", "ERROR");
            //     }
            //     $offset += 500;
            // } while ($resp["paging"]["total"] > 0);

            if (count($products) > 0) {
                
                include_once($_ENV["EDICONNECT_EBP"]."Models/ProductsModel.php");

                $functions = new Functions($this->access_token, $this->ebp_subscription_key);
                $ProductModel = new ProductModel();

                $ProductModel->exportAll($products);
            }   

        } catch (\Throwable $th) {
            //throw $th;
            $date = date("YmdHis");
    
            $error_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
            $file = fopen($error_path, "a");
            $logs->addLog($file, "EXPORT PRODUIT", $th->getMessage(), "ERROR");

        }
    } 


    public function Export($productId){

        try {
            $url = $this->APIAuth->GetApiPathProductExport($productId);
            $curl = curl_init();
    
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => "GET",
                CURLOPT_HTTPHEADER => array(
                    "ebp-subscription-key: $this->ebp_subscription_key",
                    "Authorization: Bearer $this->access_token"
                ),
            ));
    
            $resp = curl_exec($curl);
    
            $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);


            if($http_code == "200"){ 
                include_once($_ENV["EDICONNECT_EBP"]."Models/ProductsModel.php");

                $functions = new Functions($this->access_token, $this->ebp_subscription_key);
                $ProductModel = new ProductModel();
                $result = json_decode($resp,true);

                return $ProductModel->Export($result);
            }else{ 
                echo "Error : ".$http_code." <br> Please check your access TOKEN KEY's to the API.";
                return false;
            }
            curl_close($curl);
        } catch (\Throwable $th) {
            $date = date("YmdHis");
    
            $target_path = $_ENV["EDICONNECT_EBP"]."Logs/Products/Exports/Errors/ERROR_PRODUCT_$date.txt";
            $file = fopen($target_path, "a");
            $logs->addLog($file, "EXPORT PRODUIT", $th->getMessage(), "ERROR");
            return false;
        }

    } 

    public function Import(){      
        

        $url = $this->APIAuth->GetApiPathProductsImport();

        $functions = new Functions($this->access_token, $this->ebp_subscription_key);
        $logs = new Logs();

        $this_dir = dirname(__FILE__);
        $parent_dir = realpath($this_dir . '/..');
        $import_dir = $_ENV["EDICONNECT_EBP"]."Imports/Products";
        $files = glob("$import_dir/*.csv");        

        foreach($files as $filepath) {

            if ($handle = fopen($filepath, "r")) {
                $first_line_data = fgetcsv($handle,1000,";");
                $client_ref = $first_line_data[0];


                $curl = curl_init();
                curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => array('file'=> new CURLFILE($filepath)),
                CURLOPT_HTTPHEADER => array(
                        "ebp-subscription-key: $this->ebp_subscription_key",
                        "Authorization: Bearer $this->access_token"
                    ),
                ));

                $response = curl_exec($curl);
                $response = json_decode($response);

                if ($importId = $response->importId) {
                    do {
                        $importInfo = $functions->getImportInfo($importId, $this->access_token, $this->ebp_subscription_key);
                    } while ($importInfo->status === "Starting" || $importInfo->status === "Waiting");

                    $date = date("YmdHis");
                    $nb_errors = $importInfo->importInfos->itemError;

                    if ($importInfo->status === "Completed") {
                        if ($nb_errors == 0) {
                            $target_path = $_ENV["EDICONNECT_EBP"] . "Logs/Products/Imports/SUCCESS_CUSTOMER_$client_ref" . "_$date.txt";
                            $file = fopen($target_path, "a");
                            $logs->writeImportLog($file, $importInfo, "success");
                            
                            unlink($filepath);
                        }else {
                            $target_path = $_ENV["EDICONNECT_EBP"] . "Logs/Products/Imports/ERROR_CUSTOMER_$client_ref" . "_$date.txt";
                            $file = fopen($target_path, "a");
                            $logs->writeImportLog($file, $importInfo, "error");
                        }
                    }
                }
                curl_close($curl);
            }
            fclose($filepath);
        }
    } 

}
?>
