<?php

include_once("../Class/queries.class.php");
include_once("../Configurations/api.php");
include_once(dirname(__DIR__).'/vendor/autoload.php');

use Automattic\WooCommerce\Client;

$dotenv = Dotenv\Dotenv::createImmutable(dirname(__DIR__));
$dotenv->load();

class Functions
{
    public function __construct() {


        $this->APIAuth = new APIAuth();
        $this->access_token = $this->APIAuth->access_token;
        $this->ebp_subscription_key = $this->APIAuth->ebp_subscription_key;

    }

    public function getProductIdFromBarcode($access_token, $ebp_subscription_key, $product_barcode)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/".$_ENV['FOLDER_ID']."/Items/ItemsExtentedQuery?Columns=Id&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20Barcode%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$product_barcode%27",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "ebp-subscription-key: $ebp_subscription_key",
            "Authorization: Bearer $access_token"
        ),
        ));

        $response = json_decode(curl_exec($curl));

        curl_close($curl);

        if ($response->paging->returned > 0) {
            $product_id = $response->results[0]->Id;
            return $product_id;
        }else{
            return "";
        }
    }

    public function getClientIdFromGln($access_token, $ebp_subscription_key, $gln)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => "https://api-developpeurs.ebp.com/gescom/api/v1/Folders/".$_ENV['FOLDER_ID']."/Clients/ExtendedQuery?CustomerType=null&Columns=Id&WhereCondition=%20%20%20%20%20%20type%3A%20customFilter%0A%20%20%20%20%20%20column%3A%20Id%0A%20%20%20%20%20%20operator%3A%20equal%0A%20%20%20%20%20%20valueType%20%3A%20string%0A%20%20%20%20%20%20value%3A%0A%20%20%20%20%20%20-%20%27$gln%27",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "ebp-subscription-key: $ebp_subscription_key",
            "Authorization: Bearer $access_token"
        ),
        ));

        $response = json_decode(curl_exec($curl));

        curl_close($curl);

        if ($response->paging->returned > 0) {
            $client_id = $response->results[0]->Id;
            return $client_id;
        }else{
            return "";
        }
    }

    function replace_fields($file_import_path, $document_ref) {   

        
        $file_import = fopen($file_import_path, 'r');
        $lines = array();
        while ( ! feof($file_import) ) {
            $lines[] = fgets($file_import);       
        }
        fclose($file_import);

        for ( $i = 0; $i < count($lines); $i++ ) {
            $line = $lines[$i];
            $line_details = explode(',', $line); 

            if ($line_details[0] !== "") {
                $product_barcode = $line_details[6];
                $gln_client = $line_details[3];
    

                $article_code = $this->getProductIdFromBarcode($this->access_token, $this->ebp_subscription_key, $product_barcode);
                $code_client  = $this->getClientIdFromGln($this->access_token, $this->ebp_subscription_key, $gln_client);
                
                if ($code_client === "") {
                    $customerFromWordPress = $this->getCustomerFromWordpress($gln_client);

                    if ($customerFromWordPress && $customerFromWordPress->role == "customer") {
                        $filepath = $this->generateCustomerFile($customerFromWordPress);
                        if ($filepath && file_exists($filepath)) {
                            include_once("Controllers/CustomersController.php");
                            $new_customer = new CustomersController();
                            $new_customer->Import();
                        }
                    }                    
                }
                if ($article_code === "" || $code_client === "") {
                    die("Le code client ou le code produit n'a pas été trouvé.");
                }
    
                $line_details[6] = $article_code;
                $line_details[3] = $code_client;
                $retro_line = implode(',', $line_details);
                $lines[$i] = $retro_line;
            }
        }
        
        unlink($file_import_path);
        

        // create new file
        $new_file = fopen($file_import_path, 'w');
        for ($i = 0; $i < count($lines); $i++  ) {
            fputs($new_file, $lines[$i]);
        }

        fclose($new_file);
        
        return true;
    }

    function checkIfCustomerExist($file_import_path, $document_ref) {   

        
        $file_import = fopen($file_import_path, 'r');
        $lines = array();
        while ( ! feof($file_import) ) {
            $lines[] = fgets($file_import);       
        }
        fclose($file_import);

        for ( $i = 0; $i < count($lines); $i++ ) {
            $line = $lines[$i];
            $line_details = explode(',', $line); 

            if ($line_details[0] !== "") {
                $product_barcode = $line_details[6];
                $gln_client = $line_details[3];
    

                // $article_code = $this->getProductIdFromBarcode($this->access_token, $this->ebp_subscription_key, $product_barcode);
                $code_client  = $this->getClientIdFromGln($this->access_token, $this->ebp_subscription_key, $gln_client);

                if ($code_client === "") {
                    // $customerFromWordPress = $this->getCustomerFromWordpress($gln_client);

                    // if ($customerFromWordPress && $customerFromWordPress->role == "customer") {
                    //     $filepath = $this->generateCustomerFile($customerFromWordPress);
                    //     if ($filepath && file_exists($filepath)) {
                    //         include_once("Controllers/CustomersController.php");
                    //         $new_customer = new CustomersController();
                    //         $new_customer->Import();
                    //     }
                    // }else{
                    //     return false;
                    // }     
                    return false;               
                }else {
                    return true;
                }
            }
        }
        return true;
    }

    
    public function getImportInfo($importId, $access_token, $ebp_subscription_key)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => "https://api-developpeurs.ebp.com/import/api/v1/StatutResultImport?importId=$importId",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'GET',
        CURLOPT_HTTPHEADER => array(
                "ebp-subscription-key: $ebp_subscription_key",
                "Authorization: Bearer $access_token"
            ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        return json_decode($response);

    }

    public function getCustomerFromWordpress($customerId)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => "http://93.113.207.213/freshxpress/wordpress_api/customer/customer.php?id=$customerId",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = json_decode(curl_exec($curl));

        curl_close($curl);

        if (!property_exists($response, "error")) {
            return $response;
        }else {
            return null;
        }
    }

    public function generateCustomerFile($customer)
    {
        try {
            $customerId = $customer->id;
            $dateCreated = date("d/m/y", strtotime($customer->date_created));
            $customerEmail = $customer->email;
            $customerName = $customer->first_name . ' ' . $customer->last_name;

            //billing
            $customerBillingFirstName = $customer->billing->first_name;
            $customerBillingLastName = $customer->billing->last_name;
            $customerBillingAddress = $customer->billing->address_1;
            $customerBillingPostcode = $customer->billing->postcode;
            $customerBillingCity = $customer->billing->city;
            $customerBillingCountry = $customer->billing->country;
            $customerBillingPhone = $customer->billing->phone ? $customer->billing->phone : "00 00 00 00 00";
            $customerBillingEmail = $customer->billing->email;

            //shipping
            $customerShippingFirstName = $customer->shipping->first_name;
            $customerShippingLastName = $customer->shipping->last_name;
            $customerShippingAddress = $customer->shipping->address_1;
            $customerShippingPostcode = $customer->shipping->postcode;
            $customerShippingCity = $customer->shipping->city;
            $customerShippingCountry = $customer->shipping->country;
            $customerShippingPhone = $customer->shipping->phone ? $customer->shipping->phone : $customerBillingPhone;
            $customerShippingEmail = $customer->shipping->email ? $customer->shipping->email : $customerBillingEmail;

            $current_date = date('dmyHis');

            $filepath = "/var/www/html/freshxpress/EDIConnect_EBP/Imports/Customers/";
            $filename = "customer_$customerId-$current_date.csv";
            $file_import_path = $filepath . $filename;

            $new_file = fopen($file_import_path, 'w');


            $file_line = "$customerId;$dateCreated;$customerEmail;$customerName;billing;$customerBillingFirstName;$customerBillingLastName;$customerBillingEmail;$customerBillingPhone;$customerBillingAddress;$customerBillingCity;$customerBillingPostcode;$customerBillingCountry;shipping;$customerShippingFirstName;$customerShippingLastName;$customerShippingEmail;$customerShippingPhone;$customerShippingAddress;$customerShippingCity;$customerShippingPostcode;$customerShippingCountry";

            fputs($new_file, $file_line);
            fclose($new_file);

            return $file_import_path;
        } catch (\Throwable $th) {
            return false;
        }
    }


    public function checkFileSuccess($repo, $dir, $success, $tableName, $itemField, $itemId, $sysModifiedDate, $active)
    {
        $dir = $repo."$dir/";
        $filesArray = dir($dir);
        $errors = 0;
 

        while (false !== ($file = $filesArray->read())) {
            if (preg_match("/$itemId/",$file)) {
                if ($success) {
                    $queries = new Queries();
                    $isItemExistOnDb = $queries->checkItemExist($tableName, $itemField, $itemId);
                    $result = null;

                    if ($isItemExistOnDb) {
                        $result = $queries->updateSysModifiedDateItem($tableName, $itemField, $itemId, $sysModifiedDate, $active);
                    }else{
                        $result = $queries->addItem($tableName, $itemField, $itemId, $sysModifiedDate, $active);
                    }

                    if ($result) {
                        unlink("$dir/$file");
                    }else {
                        rename("$dir/$file", "$dir/Errors/$file");
                        $errors++;
                    }
                }else {
                    rename("$dir/$file", "$dir/Errors/$file");
                    $errors++;
                }
            }
        }
        $filesArray->close();

        if ($errors == 0) {
            return [
                [
                    "success" => true,
                    "message" => "La table $tableName a été mise à jour où le champ $itemField=$itemId"
                ]
            ];
        }else{
            return [
                [
                    "success" => false,
                    "message" => "La table $tableName n'a pas été mise à jour où le champ $itemField=$itemId"
                ]
            ];
        }
    }

    public function deleteFromWoocommerce($productId)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => "http://93.113.207.213/freshxpress/wordpress_api/products/delete.php?id=$productId&action=private",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = json_decode(curl_exec($curl));

        curl_close($curl);

        if (!property_exists($response, "error")) {
            return $response;
        }else {
            return json_decode(json_encode([
                "success" => false,
                "message" => "La requete 'http://93.113.207.213/freshxpress/wordpress_api/products/delete.php?id=$productId&action=private' n'a pas pu aboutir."
            ]));
        }
    }
}
?>
